@@ -8,18 +8,25 @@
 #'
 #' @param Y observations of the outcome variable. Either a numeric vector of length n
 #' or a numeric matrix with dimension n by 1.
+#' If outcome variable is binary use dummy encoding.
 #' @param D observations of the treatment variable. Either a numeric vector of length n
 #' or a numeric matrix with dimension n by 1.
-#' @param Z observations of the instrumental variable(s). Either a numeric vector of length n
-#' or a numeric matrix with dimension n by s.
-#' @param X observations of baseline covariate(s) used to fit the treatment model. Either a numeric vector of length n
-#' or a numeric matrix with dimension n by p or \code{NULL}
+#' If treatment variable is binary use dummy encoding.
+#' @param Z observations of the instrumental variable(s). Either a vector of length n
+#' or a matrix with dimension n by s.
+#' If observations are not numeric dummy encoding will be applied.
+#' @param X observations of baseline covariate(s). Either a vector of length n
+#' or a matrix with dimension n by p or \code{NULL}
 #' (if no covariates should be included).
-#' @param W (transformed) observations of baseline covariate(s) used to fit the outcome model. Either a numeric vector of length n
-#' or a numeric matrix with dimension n by p_w or \code{NULL}
+#' If observations are not numeric dummy encoding will be applied.
+#' @param W (transformed) observations of baseline covariate(s) used to fit the outcome model. Either a vector of length n
+#' or a matrix with dimension n by p_w or \code{NULL}
 #' (if no covariates should be included).
-#' @param vio_space  list with numeric vectors of length n and/or numeric matrices with n rows as elements to
-#' specify the violation space candidates. See Details for more information.
+#' If observations are not numeric dummy encoding will be applied.
+#' @param vio_space  list with vectors of length n and/or matrices with n rows as elements to
+#' specify the violation space candidates.
+#' If observations are not numeric dummy encoding will be applied.
+#' See Details for more information.
 #' @param create_nested_sequence logical. If \code{TRUE}, the violation space candidates (in form of matrices)
 #' are defined sequentially starting with an empty violation matrix and subsequently
 #' adding the next element of \code{vio_space} to the current violation matrix.
@@ -28,35 +35,38 @@
 #' @param sel_method The selection method used to estimate the treatment effect. Either "comparison" or "conservative". See Details.
 #' @param split_prop proportion of observations used to fit the outcome model. Has to be a value in (0, 1).
 #' @param nrounds number of boosting iterations.
-#' Can either be a single integer value or a vector containing multiple integer values to try.
+#' Can either be a single integer value or a vector of integer values to try.
 #' @param eta learning rate of the boosting algorithm.
-#' Can either be a single numeric value or a vector containing multiple numeric values to try.
+#' Can either be a single numeric value or a vector of numeric values to try.
 #' @param max_depth maximal tree depth.
-#' Can either be a single integer value or a vector containing multiple integer values to try.
+#' Can either be a single integer value or a vector of integer values to try.
 #' @param subsample subsample ratio of the training instance.
-#' Can either be a single numeric value or a vector containing multiple numeric values to try.
-#' Has to be a value in (0, 1].
+#' Can either be a single numeric value or a vector of numeric values to try.
+#' Has to be a numeric value in (0, 1].
 #' @param colsample_bytree subsample ratio of columns when constructing each tree.
-#' Can either be a single numeric value or a vector containing multiple numeric values to try.
-#' Has to be a value in (0, 1].
+#' Can either be a single numeric value or a vector of numeric values to try.
+#' Has to be a numeric value in (0, 1].
 #' @param early_stopping logical. If \code{TRUE}, early stopping will be applied to
 #' choose the optimal number of boosting iteration using the cross-validation mean squared error.
-#' @param nfolds number of folds used for cross-validation to choose best parameter combination.
-#' @param iv_threshold minimal value of the threshold of IV strength test.
+#' @param nfolds a positive integer value specifying the number of folds used for cross-validation to choose best parameter combination.
+#' @param self_predict logical. If \code{FALSE}, it sets the diagonal of the hat matrix
+#' of each tree to zero to avoid self-prediction and rescales the off-diagonal elements accordingly.
+#' @param sd_boot logical. if \code{TRUE}, it determines the standard error using a bootstrap approach.
+#' @param iv_threshold a numeric value specifying the minimum of the threshold of IV strength test.
 #' @param threshold_boot logical. if \code{TRUE}, it determines the threshold of the IV strength using a bootstrap approach.
 #' If \code{FALSE}, it does not perform a bootstrap. See Details.
-#' @param alpha the significance level.
+#' @param alpha the significance level. Has to be a numeric value between 0 and 1.
 #' @param nsplits number of times the data will be split. Has to be an integer larger or equal 1. See Details.
 #' @param mult_split_method method to calculate the standard errors, p-values and to construct the confidence intervals if multi-splitting is performed.
 #' Default is "DML" if \code{nsplits} == 1 and "FWER" otherwise. See Details.
 #' @param intercept logical. If \code{TRUE}, an intercept is included in the outcome model.
 #' @param parallel one out of \code{"no"}, \code{"multicore"}, or \code{"snow"} specifying the parallelization method used.
-#' @param ncores the number of cores to use.
+#' @param ncores the number of cores to use. Has to be an integer value larger or equal 1.
 #' @param cl either a parallel or snow cluster or \code{NULL}.
 #' @param raw_output logical. If \code{TRUE}, the coefficient and standard error estimates of each split will be returned.
 #' This is only needed for the use of the function \code{confint} if \code{mult_split_method} equals "FWER". Default is
 #' \code{TRUE} if \code{mult_split_method} is \code{TRUE} and \code{FALSE} otherwise.
-#' @param B number of bootstrap samples.
+#' @param B number of bootstrap samples. Has to be a positive integer value.
 #' Bootstrap methods are used to calculate the IV strength threshold if \code{threshold_boot} is \code{TRUE} and for the violation space selection.
 #'
 #' @return
@@ -129,8 +139,8 @@
 #' The instrumental variable(s) are considered strong enough for violation space candidate \eqn{V_q} if the estimated IV strength using this
 #' violation space candidate is larger than the obtained value of the threshold of the IV strength.
 #' The formula of the threshold of the IV strength has the form
-#' \eqn{\max \{ 2 \cdot \mathrm{Trace} [ \mathrm{M} (V_q) ], \mathrm{iv{\_}threshold} \} + S (V_q) } if \code{threshold_boot} is \code{TRUE}, and
-#' \eqn{\max \{ 2 \cdot \mathrm{Trace} [ \mathrm{M} (V_q) ], \mathrm{iv{\_}threshold} \}} if \code{threshold_boot} is \code{FALSE}. The matrix
+#' \eqn{\min \{\max \{ 2 \cdot \mathrm{Trace} [ \mathrm{M} (V_q) ], \mathrm{iv{\_}threshold} \} + S (V_q), 40 \}} if \code{threshold_boot} is \code{TRUE}, and
+#' \eqn{\min \{\max \{ 2 \cdot \mathrm{Trace} [ \mathrm{M} (V_q) ], \mathrm{iv{\_}threshold} \}, 40 \}} if \code{threshold_boot} is \code{FALSE}. The matrix
 #' \eqn{\mathrm{M} (V_q)} depends on the hat matrix obtained from estimating \eqn{f(Z_i, X_i)}, the violation space candidate \eqn{V_q} and
 #' the variables to include in the outcome model \code{W}. \eqn{S (V_q)} is obtained using a bootstrap and aims to adjust for the estimation error
 #' of the IV strength.
@@ -161,7 +171,8 @@
 #' pseudo random number generator stream.
 #' Thus, the results reproducible if the arguments remain unchanged.
 #' There is an optional argument \code{cl} to specify a custom cluster
-#' if \code{parallel = "snow"}.
+#' if \code{parallel = "snow"}. \cr \cr
+#' See also Carl et al. (2023) for more details.
 #'
 #' @references
 #' \itemize{
@@ -175,6 +186,9 @@
 #' Whitney Newey, and James Robins. Double/debiased machine learning for treatment
 #' and structural parameters: Double/debiased machine learning.
 #' \emph{The Econometrics Journal}, 21(1), 2018. 4, 16, 18}
+#' \item{David Carl, Corinne Emmenegger, Peter Buehlmann, and Zijian Guo. TSCI:
+#' two stage curvature identification for causal inference with invalid instruments.
+#' \emph{arXiv:2304.00513}, 2023}
 #' }
 #'
 #' @seealso
@@ -289,17 +303,32 @@ tsci_boosting <- function(Y,
                           colsample_bytree = 1,
                           early_stopping = TRUE,
                           nfolds = 5,
+                          self_predict = FALSE,
+                          sd_boot = TRUE,
                           iv_threshold = 10,
                           threshold_boot = TRUE,
                           alpha = 0.05,
                           intercept = TRUE,
-                          parallel = "no",
+                          parallel = c("no", "multicore", "snow"),
                           nsplits = 10,
                           mult_split_method = c("FWER", "DML"),
                           ncores = 1,
                           cl = NULL,
                           raw_output = NULL,
                           B = 300) {
+  # encodes categorical variables to dummy variables.
+  ls_encoded <- dummy_encoding(Y = Y,
+                               D = D,
+                               Z = Z,
+                               X = X,
+                               W = W,
+                               vio_space = vio_space)
+  Y <- ls_encoded$Y
+  D <- ls_encoded$D
+  Z <- ls_encoded$Z
+  X <- ls_encoded$X
+  W <- ls_encoded$W
+  vio_space <- ls_encoded$vio_space
 
   # checks that input is in the correct format.
   check_input(Y = Y,
@@ -318,10 +347,11 @@ tsci_boosting <- function(Y,
               colsample_bytree = colsample_bytree,
               early_stopping = early_stopping,
               nfolds = nfolds,
+              self_predict = self_predict,
+              sd_boot = sd_boot,
               iv_threshold = iv_threshold,
               threshold_boot = threshold_boot,
               alpha = alpha,
-              parallel = parallel,
               nsplits = nsplits,
               ncores = ncores,
               cl = cl,
@@ -336,17 +366,13 @@ tsci_boosting <- function(Y,
   }
   mult_split_method <- match.arg(mult_split_method)
   sel_method <- match.arg(sel_method)
+  parallel <- match.arg(parallel)
   # if TRUE returns the estimate of the treatment effect and its standard error
   # for each data split. Is needed for the confint method if mult_split_method is "FWER".
   if (is.null(raw_output)) {
     raw_output <- ifelse(mult_split_method == "FWER", TRUE, FALSE)
   }
 
-  # stores variables as matrices as matrix multiplications will be performed later.
-  Y = as.matrix(Y); D = as.matrix(D); Z = as.matrix(Z)
-  if (!is.null(X)) X <- as.matrix(X)
-  if (!is.null(W)) W <- as.matrix(W)
-
   n <- NROW(Y)
   p <- NCOL(Z) + ifelse(is.null(X), 0, NCOL(X))
 
@@ -355,12 +381,13 @@ tsci_boosting <- function(Y,
 
   # sets up grid search over the hyperparameter combinations.
   params_grid <- expand.grid(
-    nrounds = nrounds,
+    nrounds = as.integer(nrounds),
     eta = eta,
-    max_depth = max_depth,
+    max_depth = as.integer(max_depth),
     subsample = subsample,
     colsample_bytree = colsample_bytree,
     early_stopping = early_stopping,
+    self_predict = self_predict,
     lambda = 0
   )
 
@@ -384,6 +411,7 @@ tsci_boosting <- function(Y,
                              create_nested_sequence = create_nested_sequence,
                              intercept = intercept,
                              sel_method = sel_method,
+                             sd_boot = sd_boot,
                              iv_threshold = iv_threshold,
                              threshold_boot = threshold_boot,
                              alpha = alpha,
@@ -392,12 +420,12 @@ tsci_boosting <- function(Y,
                              split_prop = split_prop,
                              parallel = parallel,
                              do_parallel = do_parallel,
-                             nsplits = nsplits,
-                             ncores = ncores,
+                             nsplits = as.integer(nsplits),
+                             ncores = as.integer(ncores),
                              mult_split_method = mult_split_method,
                              cl = cl,
                              raw_output = raw_output,
-                             B = B)
+                             B = as.integer(B))
 
   # returns output.
   outputs <- append(outputs,
@@ -406,7 +434,8 @@ tsci_boosting <- function(Y,
                          n_A2 = n_A2,
                          nsplits = nsplits,
                          mult_split_method = mult_split_method,
-                         alpha = alpha))
+                         alpha = alpha,
+                         sel_method = sel_method))
   class(outputs) <- c("tsci", "list")
   return(outputs)
 }
